# coding: utf-8

import os
import re

from recycle.config import TRASH_PATH, TRASH_REGEX
from recycle.lib import (
    mkdir,
    my_print,
    operations,
    search_files,
    replace_file,
    execute_move,
    directory_exists,
    remove_trash_path,
    remove_empty_dir,
)


def recover_by_id(file_path, recover_path, copy=False):
    if replace_file(recover_path):
        mkdir(os.path.dirname(recover_path))
        execute_move(file_path, recover_path, copy)
        return remove_empty_dir(os.path.dirname(file_path))


def recover_by_regrex(absolute_dir, file_regex, recover_path, reverse):
    for absolute_trash_file_dir in search_files(absolute_dir, file_regex):
        recover_trash_file_dir = remove_trash_path(absolute_trash_file_dir)
        if not directory_exists(absolute_trash_file_dir):
            return
        trash_files_list = search_files(absolute_trash_file_dir, TRASH_REGEX)
        for file_path in sorted(trash_files_list, reverse=reverse):
            recover_by_id(file_path, recover_trash_file_dir)
            break
        remove_empty_dir(absolute_trash_file_dir)


def recover_from_trash(trash_dir, file_regex, reverse, raw):
    relative_dir = remove_trash_path(trash_dir)
    trash_id_in_dir = re.search(TRASH_REGEX, trash_dir)
    is_trash_id = re.match(TRASH_REGEX, file_regex)
    relative_dir = trash_dir.strip("/")
    absolute_dir = os.path.join(TRASH_PATH, relative_dir)
    recover_path = "/" + relative_dir

    if not directory_exists(absolute_dir):
        if directory_exists(raw):
            absolute_dir = raw
        else:
            return
    current_dir = os.getcwd()
    if is_trash_id:
        recover_by_id(os.path.join(absolute_dir, file_regex), recover_path)
        if trash_dir == current_dir:
            my_print("\nPlease run: \n\n\tcd ..;cd -")
        return
    elif trash_id_in_dir:
        recover_path = os.path.join(re.sub(TRASH_REGEX, "", recover_path), file_regex)
        recover_by_id(os.path.join(absolute_dir, file_regex), recover_path, copy=True)
        if trash_dir == current_dir:
            my_print("\nPlease run: \n\n\tcd ..;cd -")
        return
    recover_by_regrex(absolute_dir, file_regex, recover_path, reverse)


def main():
    for parent_dir, file_regex, reverse, raw in operations():
        recover_from_trash(parent_dir, file_regex, reverse, raw)



# coding: utf-8

import json
import os
import sys

HOME = os.path.expanduser("~")
INIT_TRASH_PATH = HOME + "/.Trash"
TRASH_DATETIME_FORMAT = "%Y-%m-%d_%H:%M:%S%f_"
TRASH_REGEX = "\d{4}-\d{2}-\d{2}_\d{2}:\d{2}:\d{8}_[0-9\.]+[A-Z]?"
CONFIG_PATH = HOME + "/.py_recycle.json"
if not os.path.isfile(CONFIG_PATH):
    open(CONFIG_PATH, "w").write("{}")
config = json.loads(open(CONFIG_PATH).read())

if sys.version_info[0] >= 3:
    unicode = str

for k, v in config.items():
    if isinstance(v, unicode):
        config[k] = str(v)
    elif isinstance(v, list):
        for ji, jv in enumerate(v):
            if isinstance(jv, unicode):
                config[k][ji] = str(jv)
    elif isinstance(v, dict):
        for hk, hv in v.items():
            if isinstance(hv, unicode):
                config[k][hk] = str(hv)


EMOJIS = {"file": "📄", "directory": "📁"}
COLORS = {
    "black": "0;30",
    "red": "0;31",
    "green": "0;32",
    "orange": "0;33",
    "blue": "0;34",
    "magenta": "0;35",
    "cyan": "0;36",
    "light_gray": "0;37",
    "dark_gray": "1;30",
    "light_red": "1;31",
    "light_green": "1;32",
    "yellow": "1;33",
    "light_blue": "1;34",
    "light_purple": "1;35",
    "light_cyan": "1;36",
    "white": "1;37",
}

FILE_SIZE_COLORS = {
    "file size unit": [
        "file size < 10unit",
        "file size < 100unit",
        "file size < 1000unit",
        "file size < 10000unit",
    ],
    "choice of color": list(COLORS.keys()),
    "": ["light_gray"] * 4,
    "b": ["light_gray"] * 4,
    "B": ["light_gray"] * 4,
    "K": ["light_gray"] * 3 + ["cyan"],
    "M": ["cyan", "cyan", "light_cyan", "orange"],
    "G": ["orange", "orange", "yellow", "magenta"],
    "T": ["magenta", "magenta", "light_purple", "red"],
    "P": ["red", "red", "light_red", "light_red"],
}
# User Config
ENABLE_EMOJI = config.get("ENABLE_EMOJI", True)
VERBOSE = config.get("VERBOSE", True)
ENABLE_COLOR = config.get("ENABLE_COLOR", True)
TREE_ALL_DIRECTORY_SIZE = config.get("TREE_ALL_DIRECTORY_SIZE", True)
TRASH_PATH = (config.get("TRASH_PATH") or INIT_TRASH_PATH).rstrip("/")
FILE_SIZE_COLORS = config.get("FILE_SIZE_COLORS") or FILE_SIZE_COLORS



# coding: utf-8

import os
import re
import sys
import datetime
import subprocess

from glob import glob
from recycle.config import (
    TRASH_PATH,
    TRASH_DATETIME_FORMAT,
    FILE_SIZE_COLORS,
    ENABLE_COLOR,
    COLORS,
    VERBOSE,
)

_special_chars_map = {i: "\\" + chr(i) for i in b"()[]{}?*+-|^$\\.&~# \t\n\r\v\f\"'<>`"}


def escape(pattern):
    if isinstance(pattern, str):
        return pattern.translate(_special_chars_map)
    else:
        pattern = str(pattern, "latin1")
        return pattern.translate(_special_chars_map).encode("latin1")


def remove_trash_path(trash_file_path):
    if trash_file_path.startswith(TRASH_PATH):
        return trash_file_path[len(TRASH_PATH) :]
    return trash_file_path


def mkdir(path):
    if os.path.isfile(path):
        my_print(
            "\t{} is file. Cann't create same name directory.".format(
                get_colorful_str(path, color_code="red", end="")
            )
        )
        exit(1)
    elif os.path.isdir(path):
        pass
    else:
        if VERBOSE:
            my_print("mkdir -p {}".format(path))
        os.system("mkdir -p {}".format(escape(path)))


def get_current_path():
    try:
        #return os.path.abspath(os.getcwd())
        # Support Symbolic Link
        return os.path.abspath(os.environ["PWD"])
    except Exception:
        return None


def get_colorful_str(s, end="\n", color_code=None):
    if color_code and ENABLE_COLOR:
        color_code = COLORS[color_code]
        s = "\033[{}m{}\033[0m".format(color_code, s)
    return "{}{}".format(s, end)


def my_print(s, end="\n", color_code=None):
    try:
        sys.stdout.write(get_colorful_str(s, end, color_code))
    except:
        pass


def get_size_color_code(size_str):
    index = len(size_str[:-1].split(".")[0]) - 1
    sign = size_str[-1].strip("1234567890 .")
    return FILE_SIZE_COLORS.get(sign)[index]


def get_path_size_str(file_path):
    return str(
        subprocess.check_output(["du", "-sh", file_path]).split()[0].decode("utf-8")
    )


def generate_trash_file_name(file_path):
    return datetime.datetime.now().strftime(TRASH_DATETIME_FORMAT) + get_path_size_str(
        file_path
    )


def directory_exists(directory):
    return os.path.isdir(directory) or get_all_files(directory, only_check=True)


def get_all_files(directory, only_check=False, absolute_files=False):
    files = [
        i.replace("//", "/") for i in glob(directory + "/*") + glob(directory + "/.*")
    ]
    slash_count = directory.count("/") + 1
    if only_check and files:
        return True
    if absolute_files:
        return files
    return list(map(lambda x: "/".join(x.split("/")[slash_count:]), files))


def search_files(directory, file_regex, absolute_files=True):
    if file_regex == "\.":
        return [directory]
    if not (file_regex and directory_exists(directory)):
        return []

    files = get_all_files(directory, absolute_files=absolute_files)
    tmp_files = list(filter(lambda x: x.endswith("/" + file_regex), files))
    if tmp_files:
        return tmp_files
    if file_regex == "*":
        return files
    file_compile = re.compile(file_regex)
    if absolute_files:
        match_fun = lambda x: file_compile.match(x.split("/")[-1])
    else:
        match_fun = file_compile.match
    return list(filter(match_fun, files))


def execute_delete(path):
    if VERBOSE:
        my_print("rm -rf {}".format(path))
    os.system("rm -rf {}".format(escape(path)))


def execute_dir_delete(path):
    if VERBOSE:
        my_print("rmdir {}".format(path))
    os.system("rmdir {}".format(escape(path)))


def execute_move(source, destination, copy=False):
    command = "cp -r" if copy else "mv"
    if VERBOSE:
        my_print("{} {} {}".format(command, source, destination))
    source = escape(source)
    destination = escape(destination)
    # Will change current directory to source
    os.system("{} {} {}".format(command, source, destination))


def remove_empty_dir(absolute_path):
    while not os.path.exists(absolute_path):
        absolute_path = absolute_path[: absolute_path.rindex("/")]
        if absolute_path == TRASH_PATH:
            break

    while os.path.isdir(absolute_path) and not os.listdir(absolute_path):
        execute_dir_delete(absolute_path)
        absolute_path = absolute_path[: absolute_path.rindex("/")]
        if absolute_path == TRASH_PATH:
            break


def my_input(s):
    try:
        f = raw_input
    except:
        f = input
    try:
        return f(s)
    except KeyboardInterrupt:
        exit(1)


def input_yes(s):
    result = my_input(s).lower() in ["yes", "y"]
    my_print("")
    return result


def replace_file(file_path):
    if os.path.exists(file_path):
        if input_yes(
            "\t{} already exists. Replace it? [N/y] ".format(
                get_colorful_str(file_path, color_code="red", end="")
            )
        ):
            absolute_trash_file_dir = TRASH_PATH + file_path
            mkdir(absolute_trash_file_dir)
            trash_file = os.path.join(
                absolute_trash_file_dir, generate_trash_file_name(file_path)
            )
            execute_move(file_path, trash_file)
            return True
    else:
        return True


def get_absolute_dirs(dirs):
    absolute_dirs = []

    for i in dirs:
        if i == "..":
            absolute_dirs.pop(-1)
        elif absolute_dirs and not (i or absolute_dirs[-1]):
            pass
        else:
            absolute_dirs.append(i)
    return absolute_dirs


def get_parent_dir_and_file_regex(input_arg):
    reverse = True
    if input_arg == "/":
        return "/", "\.", reverse, ""
    parent_dir = get_current_path()
    dirs = input_arg.split("/")
    raw = ""
    if dirs:
        raw = os.path.join(TRASH_PATH, "/".join(dirs[:-1]))
        if not dirs[0]:
            # use absolute path
            dirs = get_absolute_dirs(dirs)
        else:
            dirs = get_absolute_dirs(parent_dir.split("/") + dirs)
        parent_dir = "/".join(dirs[:-1]) or "/"

    file_regex = dirs[-1] if dirs else input_arg
    return parent_dir, file_regex, reverse, raw


def operations():
    for arg in sys.argv[1:]:
        if arg != "/":
            arg = arg.rstrip("/")
            arg = arg[2:] if arg[:2] == "./" else arg

        parent_dir, file_regex, reverse, raw = get_parent_dir_and_file_regex(arg)
        if not arg:
            continue
        yield parent_dir, file_regex, reverse, raw



# coding: utf-8

import os
import re

from recycle.config import TRASH_PATH, TRASH_REGEX
from recycle.lib import (
    input_yes,
    operations,
    search_files,
    execute_delete,
    directory_exists,
    remove_empty_dir,
    get_colorful_str,
    remove_trash_path,
)


def delete_by_id(absolute_file):
    return execute_delete(absolute_file)


def batch_delete(files):
    for i in files:
        execute_delete(i)
        remove_empty_dir(i)


def delete_by_regrex(absolute_file, absolute_dir, file_regex, reverse):
    deleted_files = []
    for absolute_trash_file_dir in search_files(absolute_dir, file_regex):
        if not directory_exists(absolute_trash_file_dir):
            return
        for absolute_sub_file_path in search_files(
            absolute_trash_file_dir, TRASH_REGEX
        ):
            deleted_files.append(absolute_sub_file_path)
    if deleted_files and permanently_delete_ask(absolute_file):
        batch_delete(deleted_files)


def permanently_delete_ask(absolute_file):
    return input_yes(
        "\tPermanently delete {} ? [N/y] ".format(
            get_colorful_str(absolute_file.replace("\.", ""), color_code="red", end="")
        )
    )


def permanently_delete(trash_dir, file_regex, reverse, raw):
    if trash_dir.startswith(TRASH_PATH):
        relative_dir = remove_trash_path(trash_dir).strip("/")
    else:
        relative_dir = trash_dir.strip("/")
    absolute_dir = os.path.join(TRASH_PATH, relative_dir)
    if not directory_exists(absolute_dir):
        if directory_exists(raw):
            absolute_dir = raw
        else:
            return
    absolute_file = os.path.join(absolute_dir, file_regex)
    if re.match(TRASH_REGEX, file_regex) and permanently_delete_ask(absolute_file):
        return delete_by_id(absolute_file)
    delete_by_regrex(absolute_file, absolute_dir, file_regex, reverse)
    return remove_empty_dir(absolute_dir)


def main():
    for parent_dir, file_regex, reverse, raw in operations():
        permanently_delete(parent_dir, file_regex, reverse, raw)
